@@ -177,6 +177,18 @@ magnifier_x_error_handler (Display	 *display,
 	return 0;
 }
 
+static gboolean
+can_open_display (gchar *display_name)
+{
+    Display *d;
+    if (d = XOpenDisplay (display_name))
+    {
+	XCloseDisplay (d);
+	return TRUE;
+    }
+    return FALSE;
+}
+
 static void
 magnifier_warp_cursor_to_screen (Magnifier *magnifier)
 {
@@ -184,20 +196,23 @@ magnifier_warp_cursor_to_screen (Magnifier *magnifier)
 	unsigned int mask;
 	Window root_return, child_return;
 
-	if (!XQueryPointer (GDK_DISPLAY_XDISPLAY (magnifier->source_display), 
-			    GDK_WINDOW_XWINDOW (magnifier->priv->root), 
-			    &root_return,
-			    &child_return,
-			    &x, &y,
-			    &unused_x, &unused_y,
-			    &mask))
+	if (magnifier->source_display)
 	{
+	    if (!XQueryPointer (GDK_DISPLAY_XDISPLAY (magnifier->source_display), 
+				GDK_WINDOW_XWINDOW (magnifier->priv->root), 
+				&root_return,
+				&child_return,
+				&x, &y,
+				&unused_x, &unused_y,
+				&mask))
+	    {
 		XWarpPointer (GDK_DISPLAY_XDISPLAY (magnifier->source_display),
 			      None,
 			      GDK_WINDOW_XWINDOW (magnifier->priv->root),
 			      0, 0, 0, 0,
 			      x, y);
 		XSync (GDK_DISPLAY_XDISPLAY (magnifier->source_display), FALSE);
+	    }
 	}
 }
 
@@ -389,6 +404,8 @@ magnifier_cursor_notification_init (Magnifier *magnifier)
     Display *dpy;
     Window rootwin;
 
+    if (!magnifier->source_display) return FALSE;
+
     dpy = GDK_DISPLAY_XDISPLAY (magnifier->source_display);
     cursor_client_connection = XOpenDisplay (0);
     rootwin = GDK_WINDOW_XWINDOW (magnifier->priv->root);
@@ -520,12 +537,12 @@ magnifier_realize (GtkWidget *widget)
 GdkWindow*
 magnifier_get_root (Magnifier *magnifier)
 {
-	if (!magnifier->priv->root) {
-		magnifier->priv->root = gdk_screen_get_root_window (
-			gdk_display_get_screen (magnifier->source_display,
-						magnifier->source_screen_num));
-	}
-	return magnifier->priv->root;
+    if (!magnifier->priv->root && magnifier->source_display) {
+	magnifier->priv->root = gdk_screen_get_root_window (
+	    gdk_display_get_screen (magnifier->source_display,
+				    magnifier->source_screen_num));
+    }
+    return magnifier->priv->root;
 }
 
 static gint
@@ -649,6 +666,9 @@ magnifier_reparent_zoom_regions (Magnifier *magnifier)
 static void
 magnifier_init_display (Magnifier *magnifier, gchar *display_name, gboolean is_target)
 {
+    if (!can_open_display (display_name))
+	return;
+
     if (is_target)
     {
 	magnifier->target_screen_num =
@@ -657,6 +677,8 @@ magnifier_init_display (Magnifier *magnifier, gchar *display_name, gboolean is_t
 					  NULL);
 	magnifier->target_display =
 	    gdk_display_open (display_name);
+	if (magnifier->target_display_name) g_free (magnifier->target_display_name);
+	magnifier->target_display_name = g_strdup (display_name);
 	magnifier->priv->root =
 	    gdk_screen_get_root_window (
 		gdk_display_get_screen (
@@ -671,11 +693,16 @@ magnifier_init_display (Magnifier *magnifier, gchar *display_name, gboolean is_t
 					  NULL);
 	magnifier->source_display =
 	    gdk_display_open (display_name);
-	magnifier->priv->root =
-	    gdk_screen_get_root_window (
-		gdk_display_get_screen (
-		    magnifier->source_display,
-		    magnifier->source_screen_num));
+	if (magnifier->source_display)
+	{
+	    if (magnifier->source_display_name) g_free (magnifier->source_display_name);
+	    magnifier->source_display_name = g_strdup (display_name);
+	    magnifier->priv->root =
+		gdk_screen_get_root_window (
+		    gdk_display_get_screen (
+			magnifier->source_display,
+			magnifier->source_screen_num));
+	}
     }
 }
 
@@ -1033,6 +1060,12 @@ magnifier_get_property (BonoboPropertyBag *bag,
 	case MAGNIFIER_CROSSWIRE_COLOR_PROP:
 		BONOBO_ARG_SET_LONG (arg, magnifier->crosswire_color);
 		break;
+        case MAGNIFIER_SOURCE_DISPLAY_PROP:
+		BONOBO_ARG_SET_STRING (arg, magnifier->source_display_name);
+	        break;
+	case MAGNIFIER_TARGET_DISPLAY_PROP:
+		BONOBO_ARG_SET_STRING (arg, magnifier->target_display_name);
+	        break;
 	default:
 		bonobo_exception_set (ev, ex_Bonobo_PropertyBag_NotFound);
 	};
@@ -1052,39 +1085,47 @@ magnifier_set_property (BonoboPropertyBag *bag,
 	switch (arg_id) {
 	case MAGNIFIER_SOURCE_DISPLAY_PROP:
 		full_display_string = BONOBO_ARG_GET_STRING (arg);
-		magnifier->source_screen_num =
+		if (can_open_display (full_display_string))
+		{
+		    magnifier->source_screen_num =
 			magnifier_parse_display_name (magnifier,
 						      full_display_string,
 						      NULL);
-		magnifier->source_display =
+		    magnifier->source_display =
 			gdk_display_open (full_display_string);
-		magnifier->priv->root =
+		    magnifier->source_display_name = g_strdup (full_display_string);
+		    magnifier->priv->root =
 			gdk_screen_get_root_window (
-				gdk_display_get_screen (
-					magnifier->source_display,
-					magnifier->source_screen_num));
-		magnifier_warp_cursor_to_screen (magnifier);
-		magnifier_get_display_rect_bounds (magnifier, &magnifier->source_bounds, FALSE);
-		magnifier_check_set_struts (magnifier);
+			    gdk_display_get_screen (
+				magnifier->source_display,
+				magnifier->source_screen_num));
+		    magnifier_warp_cursor_to_screen (magnifier);
+		    magnifier_get_display_rect_bounds (magnifier, &magnifier->source_bounds, FALSE);
+		    magnifier_check_set_struts (magnifier);
+		}
 		break;
 	case MAGNIFIER_TARGET_DISPLAY_PROP:
 		full_display_string = BONOBO_ARG_GET_STRING (arg);
-		magnifier->target_screen_num =
+		if (can_open_display (full_display_string))
+		{
+		    magnifier->target_screen_num =
 			magnifier_parse_display_name (magnifier,
 						      full_display_string,
 						      NULL);
-		magnifier->target_display =
+		    magnifier->target_display =
 			gdk_display_open (full_display_string);
-		if (GTK_IS_WINDOW (magnifier->priv->w)) 
+		    magnifier->target_display_name = g_strdup (full_display_string);
+		    if (GTK_IS_WINDOW (magnifier->priv->w)) 
 			gtk_window_set_screen (GTK_WINDOW (magnifier->priv->w), 
 					       gdk_display_get_screen (
-						       magnifier->target_display,
-						       magnifier->target_screen_num));
-		magnifier_get_display_rect_bounds (magnifier, &magnifier->target_bounds, TRUE);
-		magnifier_get_display_rect_bounds (magnifier, &magnifier->source_bounds, FALSE);
-		magnifier_reparent_zoom_regions (magnifier);
-		magnifier_init_cursor_set (magnifier, magnifier->cursor_set); /* needed to reset pixmaps */
-		magnifier_check_set_struts (magnifier);
+						   magnifier->target_display,
+						   magnifier->target_screen_num));
+		    magnifier_get_display_rect_bounds (magnifier, &magnifier->target_bounds, TRUE);
+		    magnifier_get_display_rect_bounds (magnifier, &magnifier->source_bounds, FALSE);
+		    magnifier_reparent_zoom_regions (magnifier);
+		    magnifier_init_cursor_set (magnifier, magnifier->cursor_set); /* needed to reset pixmaps */
+		    magnifier_check_set_struts (magnifier);
+		}
 		break;
 	case MAGNIFIER_SOURCE_SIZE_PROP:
 	        magnifier->source_bounds = BONOBO_ARG_GET_GENERAL (arg,
@@ -1182,13 +1223,13 @@ impl_magnifier_set_source_display (PortableServer_Servant servant,
 	Magnifier *magnifier = MAGNIFIER (bonobo_object_from_servant (servant));
 	BonoboArg *arg = bonobo_arg_new (BONOBO_ARG_STRING);
 	BONOBO_ARG_SET_STRING (arg, display);
-
+	
 	magnifier_set_property (magnifier->property_bag,
 				arg,
 				MAGNIFIER_SOURCE_DISPLAY_PROP,
 				ev,
 				magnifier);
-
+	
 	/* attach listeners for DAMAGE, "dirty region", XFIXES cursor changes */
 	magnifier_set_extension_listeners (magnifier, magnifier_get_root (magnifier));
 
@@ -1213,6 +1254,24 @@ impl_magnifier_set_target_display (PortableServer_Servant servant,
 	bonobo_arg_release (arg);
 }
 
+static 
+CORBA_string
+impl_magnifier_get_source_display (PortableServer_Servant servant,
+				   CORBA_Environment *ev)
+{
+        Magnifier *magnifier = MAGNIFIER (bonobo_object_from_servant (servant));
+	return CORBA_string_dup (magnifier->source_display_name ? magnifier->source_display_name : "");
+}
+
+static 
+CORBA_string
+impl_magnifier_get_target_display (PortableServer_Servant servant,
+				   CORBA_Environment *ev)
+{
+        Magnifier *magnifier = MAGNIFIER (bonobo_object_from_servant (servant));
+	return CORBA_string_dup (magnifier->source_display_name ? magnifier->target_display_name : "");
+}
+
 static GNOME_Magnifier_ZoomRegion
 impl_magnifier_create_zoom_region (PortableServer_Servant servant,
 				   const CORBA_float zx,
@@ -1355,6 +1414,8 @@ magnifier_class_init (MagnifierClass *klass)
 
         epv->_set_SourceDisplay = impl_magnifier_set_source_display;
 	epv->_set_TargetDisplay = impl_magnifier_set_target_display;
+        epv->_get_SourceDisplay = impl_magnifier_get_source_display;
+	epv->_get_TargetDisplay = impl_magnifier_get_target_display;
 	epv->getProperties = impl_magnifier_get_properties;
 	epv->getZoomRegions = impl_magnifier_get_zoom_regions;
 	epv->createZoomRegion = impl_magnifier_create_zoom_region;
@@ -1553,6 +1614,8 @@ magnifier_init (Magnifier *magnifier)
 	magnifier->zoom_regions = NULL;
 	magnifier->source_screen_num = 0;
 	magnifier->target_screen_num = 0;
+	magnifier->source_display_name = g_strdup (":0.0");
+	magnifier->target_display_name = g_strdup (":0.0");
 	magnifier->cursor_size_x = 0;
 	magnifier->cursor_size_y = 0;
 	magnifier->cursor_scale_factor = 1.0F;
