@@ -50,6 +50,8 @@ typedef struct {
 	BonoboPropertyBag *property_bag;
 	GdkDisplay *source_display;
 	GdkDisplay *target_display;
+	gchar *source_display_name;
+	gchar *target_display_name;
 	int source_screen_num;
 	int target_screen_num;
 	GList *zoom_regions;
